# -*- coding: UTF-8 -*-

"""
analyst.classify - 对交易数据进行分类分析
====================================================================
"""


class ShareTrendClassifier(object):
    """个股走势分类器"""
    pass


class IndexTrendClassifier(object):
    """指数走势分类器"""
    pass
